@@ -1429,7 +1429,7 @@ static void get_record_cb(sdp_list_t *recs, int err, gpointer user_data)
 
 	err = rfcomm_connect(dev, NULL, NULL, NULL);
 	if (err < 0) {
-		error("Unable to connect: %s (%s)", strerror(-err), -err);
+		error("Unable to connect: %s (%d)", strerror(-err), -err);
 		p->err = -err;
 		error_connection_attempt_failed(dev->conn, p->msg, p->err);
 		goto failed;
@@ -2081,6 +2081,7 @@ static void path_unregister(void *data)
 
 	if (hs->state > HEADSET_STATE_DISCONNECTED) {
 		debug("Headset unregistered while device was connected!");
+		bt_cancel_discovery(&dev->src, &dev->dst);
 		headset_set_state(dev, HEADSET_STATE_DISCONNECTED);
 	}
 
@@ -2387,7 +2388,8 @@ static int headset_close_rfcomm(struct audio_device *dev)
 	return 0;
 }
 
-static void disconnect_cb(struct btd_device *btd_dev, void *user_data)
+static void disconnect_cb(struct btd_device *btd_dev, gboolean removal,
+				void *user_data)
 {
 	struct audio_device *device = user_data;
 
