@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2006-2007  Nokia Corporation
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -36,10 +36,14 @@ void audio_manager_exit(void);
 
 gboolean server_is_enabled(bdaddr_t *src, uint16_t svc);
 
-struct audio_device *manager_find_device(const bdaddr_t *bda, const char *interface,
+struct audio_device *manager_find_device(const char *path,
+					const bdaddr_t *src,
+					const bdaddr_t *dst,
+					const char *interface,
 					gboolean connected);
 
 struct audio_device *manager_get_device(const bdaddr_t *src,
-					const bdaddr_t *dst);
+					const bdaddr_t *dst,
+					gboolean create);
 
 gboolean manager_allow_headset_connection(bdaddr_t *src);
