@@ -326,7 +326,10 @@ static void link_key_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 
 		debug("stored link key type = 0x%02x", type);
 
-		if ((type == 0x03 || type == 0x04) && (req.type & 0x01))
+		/* Don't use debug link keys (0x03) and also don't use
+		 * unauthenticated combination keys if MITM is required */
+		if (type == 0x03 || (type == 0x04 && req.type != 0xff &&
+							(req.type & 0x01)))
 			hci_send_cmd(dev, OGF_LINK_CTL,
 					OCF_LINK_KEY_NEG_REPLY, 6, dba);
 		else
@@ -345,22 +348,27 @@ static void link_key_notify(int dev, bdaddr_t *sba, void *ptr)
 	uint8_t old_key_type;
 
 	ba2str(sba, sa); ba2str(dba, da);
-	info("link_key_notify (sba=%s, dba=%s)", sa, da);
+	info("link_key_notify (sba=%s, dba=%s, type=%d)", sa, da,
+							evt->key_type);
 
 	err = read_link_key(sba, dba, old_key, &old_key_type);
 	if (err < 0)
-		old_key_type = 0xFF;
+		old_key_type = 0xff;
 
 	dev_id = hci_devid(sa);
 
-	err = write_link_key(sba, dba, evt->link_key, evt->key_type,
-						io_data[dev_id].pin_length);
+	err = hcid_dbus_link_key_notify(sba, dba, evt->link_key, evt->key_type,
+						io_data[dev_id].pin_length,
+						old_key_type);
 	if (err < 0) {
 		uint16_t handle;
 
-		error("write_link_key: %s (%d)", strerror(-err), -err);
-
-		hcid_dbus_bonding_process_complete(sba, dba, HCI_MEMORY_FULL);
+		if (err == -ENODEV)
+			hcid_dbus_bonding_process_complete(sba, dba,
+							HCI_OE_LOW_RESOURCES);
+		else
+			hcid_dbus_bonding_process_complete(sba, dba,
+							HCI_MEMORY_FULL);
 
 		if (get_handle(dev, sba, dba, &handle) == 0) {
 			disconnect_cp cp;
@@ -372,8 +380,7 @@ static void link_key_notify(int dev, bdaddr_t *sba, void *ptr)
 			hci_send_cmd(dev, OGF_LINK_CTL, OCF_DISCONNECT,
 						DISCONNECT_CP_SIZE, &cp);
 		}
-	} else
-		hcid_dbus_link_key_notify(sba, dba, evt->key_type, old_key_type);
+	}
 
 	io_data[dev_id].pin_length = -1;
 }
@@ -784,6 +791,13 @@ static inline void auth_complete(int dev, bdaddr_t *sba, void *ptr)
 	hcid_dbus_bonding_process_complete(sba, &dba, evt->status);
 }
 
+static inline void simple_pairing_complete(int dev, bdaddr_t *sba, void *ptr)
+{
+	evt_simple_pairing_complete *evt = ptr;
+
+	hcid_dbus_simple_pairing_complete(sba, &evt->bdaddr, evt->status);
+}
+
 static inline void conn_request(int dev, bdaddr_t *sba, void *ptr)
 {
 	evt_conn_request *evt = ptr;
@@ -894,6 +908,10 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 		auth_complete(dev, &di->bdaddr, ptr);
 		break;
 
+	case EVT_SIMPLE_PAIRING_COMPLETE:
+		simple_pairing_complete(dev, &di->bdaddr, ptr);
+		break;
+
 	case EVT_CONN_REQUEST:
 		conn_request(dev, &di->bdaddr, ptr);
 		break;
